package djmil.cordacheckers.gameproposal;

import java.net.URI;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.ResponseEntity;
import org.springframework.security.core.annotation.AuthenticationPrincipal;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.RestController;
import org.springframework.web.util.UriComponentsBuilder;

import djmil.cordacheckers.cordaclient.CordaClient;
import djmil.cordacheckers.cordaclient.dao.HoldingIdentity;
import djmil.cordacheckers.cordaclient.dao.flow.arguments.CreateGameProposal;
import djmil.cordacheckers.cordaclient.dao.Color;
import djmil.cordacheckers.cordaclient.dao.GameProposal;
import djmil.cordacheckers.user.HoldingIdentityResolver;
import djmil.cordacheckers.user.User;

import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;


@RestController
@RequestMapping("gameproposal")
public class GameProposalController {

    @Autowired
    CordaClient cordaClient;

    @Autowired
    HoldingIdentityResolver holdingIdentityResolver;

    @GetMapping
    public ResponseEntity<String> findAllUnconsumed(
        @AuthenticationPrincipal User player
    ) {
        String gpList = cordaClient.listGameProposals(player.getHoldingIdentity());

        return ResponseEntity.ok(gpList);
    }

    // @PostMapping()
    // public  ResponseEntity<String> gameproposalSend(@AuthenticationPrincipal ApiUserDetails user) {
        
    //     return  ResponseEntity.ok("");
    // }
    
    @PostMapping()
    public ResponseEntity<Void> createGameProposal(
        @AuthenticationPrincipal User sender,
        @RequestBody CreateGameProposal gpRequest,
        UriComponentsBuilder ucb
    ) {
        final HoldingIdentity gpSender = sender.getHoldingIdentity();
        // TODO: throw execption with custom type
        final HoldingIdentity gpReceiver = holdingIdentityResolver.getByUsername(gpRequest.opponentName());
        final Color gpReceiverColor = gpRequest.opponentColor();

        String newGameProposalUuid = cordaClient.createGameProposal(
            gpSender,
            gpReceiver,
            gpReceiverColor,
            gpRequest.message()
        );

        URI locationOfNewGameProposal = ucb
            .path("gameproposal/{id}")
            .buildAndExpand(newGameProposalUuid)
            .toUri();

        return ResponseEntity
            .created(locationOfNewGameProposal)
            .build();
    }

}