package djmil.cordacheckers.cordaclient;

import org.junit.jupiter.api.Test;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.test.context.SpringBootTest;

import com.fasterxml.jackson.core.JsonProcessingException;

import djmil.cordacheckers.cordaclient.dao.Color;
import djmil.cordacheckers.cordaclient.dao.VirtualNode;
import djmil.cordacheckers.user.HoldingIdentityResolver;

import static org.assertj.core.api.Assertions.assertThat;

import java.util.List;

import javax.naming.InvalidNameException;

@SpringBootTest
public class CordaClientTest {
    @Autowired
    CordaClient cordaClient;

    @Autowired
    HoldingIdentityResolver holdingIdentityResolver;

    @Test
    void testGetVirtualNodeList() throws InvalidNameException {
        List<VirtualNode> vNodes = cordaClient.getVirtualNodeList();

		assertThat(vNodes.size()).isEqualTo(5); // default vNode config for CSDE
    }

    @Test
    void testGameProposalList() throws JsonProcessingException {
        String resp = cordaClient.listGameProposals(
            holdingIdentityResolver.getByUsername("alice"));

        System.out.println("testListGameProposals "+ resp);
    }

    @Test
    void testGemeProposalCreate() {
        final String gpSender = "alice";
        final String gpReceiver = "bob";
        final Color gpReceiverColor = Color.WHITE;
        final String gpMessage = "GameProposal create test";

        final String gpUuid = cordaClient.createGameProposal(
            holdingIdentityResolver.getByUsername(gpSender),
            holdingIdentityResolver.getByUsername(gpReceiver),
            gpReceiverColor,
            gpMessage
        );

        String listResSender = cordaClient.listGameProposals(
            holdingIdentityResolver.getByUsername(gpSender));

        String listResReceiver = cordaClient.listGameProposals(
            holdingIdentityResolver.getByUsername(gpReceiver));

        assertThat(listResSender).contains(gpUuid);
        assertThat(listResReceiver).contains(gpUuid);
    }
}
